;; Airplane IR - Copyright 2013 by Michael Kohn
;; Email: mike to mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Balsa wood airplane controlled by a Syma S107 remote

.include "tn85def.inc"
.avr8

; In this project, we don't really care about carrier frequency, but
; let's interrupt based on it anyway.

; CKSEL = 0001
; CLKPS = 0000
; CKDIV8 = 0
;
; FUSE LOW = 11100001 = 0xe1

;  cycles  sample rate   @16 MHz:
;    210   38.0kHz * 2

; start: 2ms on / 2ms off
;         32000 / 32000         32000 / 210 = 152
;
;     0: 0.3ms on/ 0.3ms off     4800 / 210 = 23
;     1: 0.3ms on/ 0.6ms off     4800 / 210 = 23
;                                9600 / 210 = 46
;  stop: 0.3 on/ 0.3 off?

; HH YAW PTCH THROTTLE YAW_CORRECT
; YAW = 0-126  63
; PITCH = 0-126 63
; THROTTLE = 0-63  (bit 7 is A/B switch)
; CORR = 0-126 63

; r0  = 0
; r1  = 1
; r15 = 255
; r14 = temp
; r17 = temp
; r18 = temp
; r20 = count in interrupt
; r21 = current byte being read in
; r22 = byte_count
; r23 = bit_count
; r24 = PWM count current
; r25 = PWM count value
; r26 = idle counter low
; r27 = idle counter high
; r30 = Z
; r31 = Z (pointer to IR data)
;

; note: CLKSEL 10

HEADER equ 152
SHORT equ 23
LONG equ 46

.org 0x000
  rjmp start
.org 0x00a
  rjmp service_interrupt

;; FIXME - erase this padding.. it's dumb
.org 0x020

start:
  ;; I'm busy.  Don't interrupt me!
  cli

  ;; Set up stack ptr
  ;ldi r17, RAMEND>>8
  ;out SPH, r17
  ldi r17, RAMEND&255
  out SPL, r17

  ;; r0 = 0, r1 = 1, r15 = 255
  eor r0, r0
  eor r1, r1
  inc r1
  ldi r17, 0xff
  mov r15, r17

  ;; Set up PORTB
  ldi r17, 0x1e
  out DDRB, r17             ; entire port B is output except PB0
  out PORTB, r0             ; turn off all of port B

  ;; Thrust is initialized to 0, idle counter is at 0
  ldi r25, 0
  ldi r26, 0
  ldi r27, 0

  ;; Set up TIMER0
  ldi r17,  210                  ; set carrier freq
  out OCR0A, r17

  ldi r17, (1<<OCIE0A)
  out TIMSK, r17                 ; enable interrupt compare A 
  ldi r17, (1<<WGM01)
  out TCCR0A, r17                ; normal counting (0xffff is top, count up)
  ldi r17, (1<<CS00)             ; CTC OCR0A  Clear Timer on Compare
  out TCCR0B, r17                ; prescale = 1 from clock source

  ; Fine, I can be interrupted now
  sei

main:
  sbic PINB, 0   ; if (IR) break;
  rjmp main

  sbi PORTB, 3

  ;; Point Z to SRAM, so we can put the IR remote data there
  ldi r30, SRAM_START&0xff
  ldi r31, SRAM_START>>8

  ;; Check for the IR header signal (2ms on / 2ms off)
  ldi r20, 0
wait_start_bit:
  cpi r20, HEADER+10
  brsh main      ; if (r20 longer header time) abort;
  sbis PINB, 0   ; if (no IR) break;
  rjmp wait_start_bit

  cpi r20, HEADER-10
  brlo main      ; if (r20 wasn't long enough) abort;

  ldi r20, 0
wait_start_bit_off:
  cpi r20, HEADER+10
  brsh main      ; if (r20 longer header time) abort;
  sbic PINB, 0   ; if (IR) break;
  rjmp wait_start_bit_off

  ;sbi PORTB, 3
  ;; Now read in 32 bits of IR data into SRAM to SRAM + 3

  ldi r22, 4     ; byte_count = 4

read_byte:
  ldi r21, 0     ; current_byte = 0
  ldi r23, 8     ; bit_count = 8

read_bit:
  ldi r20, 0
read_bit_on:
  cpi r20, SHORT+10
  brsh main      ; if (r20 longer than short time) abort;
  sbis PINB, 0   ; if (no IR) break;
  rjmp read_bit_on

  ldi r20, 0
read_bit_off:
  cpi r20, LONG+10
  brsh main      ; if (r20 longer than long time) abort;
  sbic PINB, 0   ; if (IR) break;
  rjmp read_bit_off

  lsl r21        ; current_byte <<= 1;
  cpi r20, SHORT+5
  brlo not_a_one
  sbr r21, 1     ; if (r20 longer than a short) current_byte |= 1;
not_a_one:

  dec r23        ; bit_count--;
  brne read_bit  ; do while(bit_count != 0)

  st Z+, r21     ; SRAM[Z++] = curent_byte
  dec r22        ; byte_count--;
  brne read_byte

  ;; wait for IR to stop
  ldi r20, 0
read_last_bit_on:
  cpi r20, SHORT+10
  brsh main      ; if (r20 longer than long time) abort;
  sbis PINB, 0   ; if (IR) break;
  rjmp read_last_bit_on

  ;; We have 4 bytes now, let's process it
  cbi PORTB, 3  ;; debug

  ;; Set new value for thrust
  lds r17, SRAM_START+2
  andi r17, 0x7f   ; filter out A/B switch
  mov r25, r17
  ldi r26, 0
  ldi r27, 0

  ;; Check input to see what position control planes should be in.
  lds r17, SRAM_START+0
  cpi r17, 30
  brsh not_left
  cbi PORTB, 1      ; left off
  sbi PORTB, 2      ; right on
  rjmp main
not_left:

  cpi r17, 90
  brlo not_right
  sbi PORTB, 1      ; left on
  cbi PORTB, 2      ; right off
  rjmp main
not_right:

  lds r17, SRAM_START+1
  cpi r17, 106
  brlo not_up
  sbi PORTB, 1      ; left on
  sbi PORTB, 2      ; right on
  rjmp main
not_up:

  cbi PORTB, 1
  cbi PORTB, 2
  rjmp main

service_interrupt:
  in r7, SREG
  inc r20

  ;; Counter how long we haven't seen IR data.  For safety, if we
  ;; stop seeing IR for around 1 second, then reset the thrust and
  ;; control planes to 0.
  dec r26
  brne exit_idle_counter
  dec r27
  brne exit_idle_counter
  ldi r25, 0
  cbi PORTB, 3
  cbi PORTB, 1
  cbi PORTB, 2
exit_idle_counter:

  ;; Debug code, toggle PB1 and test on scope for 38kHz
.if 0
  inc r26
  sbrs r26, 0
  sbi PORTB, 1
  sbrc r26, 0
  cbi PORTB, 1
.endif

  ;; PWM motor
  inc r24        ; inc count_current
  cpi r24, 64    ; if (count_current < 64) not ready to reset cycle 
  brlo dont_reset
  ldi r24, 0
  tst r25        ; if (count_setting ==0) don't turn on motor
  breq dont_reset
  sbi PORTB, 4   ; turn on motor

dont_reset:
  cp r25, r24    ; if (count_setting >= count_current) dont turn off motor
  brsh dont_turn_off_motor
  cbi PORTB, 4   ; turn on motor
dont_turn_off_motor:

exit_interrupt:
  out SREG, r7
  reti

signature:
.db "IR Balsa Airplane - Copyright 2013 - Michael Kohn - Version 0.01",0



